import functools
import os
import sys
import requests
import requests.exceptions


class _out:
    def __init__(self):
        self.dsp = ''
        self.stdout = sys.stdout

    def write(self, txt):
        txt = str(txt)
        self.stdout.write(txt)
        self.dsp += txt

    def flush(self):
        self.stdout.flush()


class _base_send_class:
    def send(self, *args, **kwargs):
        pass


class NOT_SEND:
    def __init__(self, RTN_CODE=None):
        self.rtn_code = RTN_CODE


def send(cls: _base_send_class):
    def decorator(func):
        @functools.wraps(func)
        def wrapper(*args, **kw):
            baseout = sys.stdout
            sys.stdout = _out()
            isSend = True
            try:
                rtn = func(*args, **kw)
                if type(rtn) is NOT_SEND:
                    isSend = False
                    rtn = rtn.rtn_code
                return rtn
            except Exception as e:
                print(str(e))
            finally:
                isSend and sys.stdout.dsp and cls.send(sys.stdout.dsp)
                sys.stdout = baseout

        return wrapper

    return decorator


class telegram(_base_send_class):
    def __init__(self, BOT_TOKEN, CHAT_ID, TG_API_HOST=os.getenv('TG_API_HOST')):
        self.BOT_TOKEN = BOT_TOKEN
        self.CHAT_ID = CHAT_ID
        self.TG_API_HOST = TG_API_HOST if TG_API_HOST else 'https://api.telegram.org'

    def send(self, msg):
        data = (
            ('chat_id', self.CHAT_ID),
            ('text', msg)
        )
        response = requests.post(f'{self.TG_API_HOST}/bot{self.BOT_TOKEN}/sendMessage', data=data)
        if response.status_code != 200:
            return False
        else:
            return response.json()['ok']
